@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2022, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -38,6 +38,8 @@ A full directory layout might look like:
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
@@ -47,12 +49,28 @@ log = logging.getLogger(__name__)
 
 # Standard library imports
 import sys
-from os.path import basename, dirname, exists, join
+from os.path import (
+    basename,
+    dirname,
+    exists,
+    join,
+)
+from types import ModuleType
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Coroutine,
+    Dict,
+    List,
+)
 
 # External imports
-from jinja2 import Environment, FileSystemLoader
+from jinja2 import Environment, FileSystemLoader, Template
 
 # Bokeh imports
+from ...core.types import PathLike
+from ...document import Document
+from ..application import ServerContext, SessionContext
 from .code_runner import CodeRunner
 from .handler import Handler
 from .notebook import NotebookHandler
@@ -60,6 +78,11 @@ from .script import ScriptHandler
 from .server_lifecycle import ServerLifecycleHandler
 from .server_request_handler import ServerRequestHandler
 
+if TYPE_CHECKING:
+    from tornado.httputil import HTTPServerRequest
+
+    from ...themes import Theme
+
 #-----------------------------------------------------------------------------
 # Globals and constants
 #-----------------------------------------------------------------------------
@@ -79,26 +102,36 @@ __all__ = (
 class DirectoryHandler(Handler):
     ''' Load an application directory which modifies a Document.
 
+    .. autoclasstoc::
+
     '''
 
-    def __init__(self, *args, **kwargs):
+    _package_runner: CodeRunner | None
+    _package: ModuleType | None
+
+    _lifecycle_handler: Handler
+    _request_handler: Handler
+
+    _theme: Theme | None
+    _static: str | None
+    _template: Template | None
+
+    def __init__(self, *, filename: PathLike, argv: List[str] = []) -> None:
         '''
         Keywords:
             filename (str) : a path to an application directory with either "main.py" or "main.ipynb"
 
             argv (list[str], optional) : a list of string arguments to make available as sys.argv to main.py
         '''
-        super().__init__(*args, **kwargs)
+        super().__init__()
 
-        if 'filename' not in kwargs:
-            raise ValueError('Must pass a filename to DirectoryHandler')
-        src_path = kwargs['filename']
-        argv = kwargs.get('argv', [])
+        src_path = filename
 
         init_py = join(src_path, '__init__.py')
         if exists(init_py):
             self._package_runner = CodeRunner(open(init_py).read(), init_py, argv)
-            self._package =  self._package_runner.new_module()
+            self._package = self._package_runner.new_module()
+            assert self._package is not None
             sys.modules[self._package.__name__] = self._package
         else:
             self._package_runner = None
@@ -107,14 +140,14 @@ class DirectoryHandler(Handler):
         main_py = join(src_path, 'main.py')
         main_ipy = join(src_path, 'main.ipynb')
         if exists(main_py) and exists(main_ipy):
-            log.warning("Found both 'main.py' and 'main.ipynb' in %s, using 'main.py'" % (src_path))
+            log.warning(f"Found both 'main.py' and 'main.ipynb' in {src_path}, using 'main.py'")
             main = main_py
         elif exists(main_py):
             main = main_py
         elif exists(main_ipy):
             main = main_ipy
         else:
-            raise ValueError("No 'main.py' or 'main.ipynb' in %s" % (src_path))
+            raise ValueError(f"No 'main.py' or 'main.ipynb' in {src_path}")
         self._path = src_path
         self._main = main
 
@@ -132,17 +165,14 @@ class DirectoryHandler(Handler):
             hooks = app_hooks
 
         if hooks is not None:
-            self._lifecycle = hooks
-            self._lifecycle_handler = ServerLifecycleHandler(filename=self._lifecycle, argv=argv, package=self._package)
+            self._lifecycle_handler = ServerLifecycleHandler(filename=hooks, argv=argv, package=self._package)
         else:
-            self._lifecycle = None
             self._lifecycle_handler = Handler() # no-op handler
 
         if exists(app_hooks):
-            self._request_handler = hooks
-            self._request_handler = ServerRequestHandler(filename=self._request_handler, argv=argv, package=self._package)
+            assert hooks is not None
+            self._request_handler = ServerRequestHandler(filename=hooks, argv=argv, package=self._package)
         else:
-            self._request_handler = None
             self._request_handler = Handler() # no-op handler
 
         self._theme = None
@@ -164,28 +194,28 @@ class DirectoryHandler(Handler):
     # Properties --------------------------------------------------------------
 
     @property
-    def error(self):
+    def error(self) -> str | None:
         ''' If the handler fails, may contain a related error message.
 
         '''
         return self._main_handler.error or self._lifecycle_handler.error
 
     @property
-    def error_detail(self):
+    def error_detail(self) -> str | None:
         ''' If the handler fails, may contain a traceback or other details.
 
         '''
         return self._main_handler.error_detail or self._lifecycle_handler.error_detail
 
     @property
-    def failed(self):
+    def failed(self) -> bool:
         ''' ``True`` if the handler failed to modify the doc
 
         '''
         return self._main_handler.failed or self._lifecycle_handler.failed
 
     @property
-    def safe_to_fork(self):
+    def safe_to_fork(self) -> bool:
         ''' Whether it is still safe for the Bokeh server to fork new workers.
 
         ``False`` if the configured code (script, notebook, etc.) has already
@@ -196,7 +226,7 @@ class DirectoryHandler(Handler):
 
     # Public methods ----------------------------------------------------------
 
-    def modify_document(self, doc):
+    def modify_document(self, doc: Document) -> None:
         ''' Execute the configured ``main.py`` or ``main.ipynb`` to modify the
         document.
 
@@ -218,7 +248,7 @@ class DirectoryHandler(Handler):
         # This internal handler should never add a template
         self._main_handler.modify_document(doc)
 
-    def on_server_loaded(self, server_context):
+    def on_server_loaded(self, server_context: ServerContext) -> None:
         ''' Execute `on_server_unloaded`` from ``server_lifecycle.py`` (if
         it is defined) when the server is first started.
 
@@ -231,7 +261,7 @@ class DirectoryHandler(Handler):
             self._package_runner.run(self._package)
         return self._lifecycle_handler.on_server_loaded(server_context)
 
-    def on_server_unloaded(self, server_context):
+    def on_server_unloaded(self, server_context: ServerContext) -> None:
         ''' Execute ``on_server_unloaded`` from ``server_lifecycle.py`` (if
         it is defined) when the server cleanly exits. (Before stopping the
         server's ``IOLoop``.)
@@ -247,7 +277,7 @@ class DirectoryHandler(Handler):
         '''
         return self._lifecycle_handler.on_server_unloaded(server_context)
 
-    def on_session_created(self, session_context):
+    def on_session_created(self, session_context: SessionContext) -> Coroutine[Any, Any, None]:
         ''' Execute ``on_session_created`` from ``server_lifecycle.py`` (if
         it is defined) when a new session is created.
 
@@ -257,7 +287,7 @@ class DirectoryHandler(Handler):
         '''
         return self._lifecycle_handler.on_session_created(session_context)
 
-    def on_session_destroyed(self, session_context):
+    def on_session_destroyed(self, session_context: SessionContext) -> Coroutine[Any, Any, None]:
         ''' Execute ``on_session_destroyed`` from ``server_lifecycle.py`` (if
         it is defined) when a session is destroyed.
 
@@ -267,7 +297,7 @@ class DirectoryHandler(Handler):
         '''
         return self._lifecycle_handler.on_session_destroyed(session_context)
 
-    def process_request(self, request):
+    def process_request(self, request: HTTPServerRequest) -> Dict[str, Any]:
         ''' Processes incoming HTTP request returning a dictionary of
         additional data to add to the session_context.
 
@@ -280,7 +310,7 @@ class DirectoryHandler(Handler):
         '''
         return self._request_handler.process_request(request)
 
-    def url_path(self):
+    def url_path(self) -> str | None:
         ''' The last path component for the basename of the path to the
         configured directory.
 
