@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2022, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -19,6 +19,8 @@ notebook code is executed, the Document being modified will be available as
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
@@ -28,8 +30,11 @@ log = logging.getLogger(__name__)
 
 # Standard library imports
 import re
+from types import ModuleType
+from typing import List
 
 # Bokeh imports
+from ...core.types import PathLike
 from ...util.dependencies import import_required
 from .code import CodeHandler
 
@@ -53,13 +58,15 @@ class NotebookHandler(CodeHandler):
     ''' A Handler that uses code in a Jupyter notebook for modifying Bokeh
     Documents.
 
+    .. autoclasstoc::
+
     '''
 
     _logger_text = "%s: call to %s() ignored when running notebooks with the 'bokeh' command."
 
     _origin = "Notebook"
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *, filename: PathLike, argv: List[str] = [], package: ModuleType | None = None) -> None:
         '''
 
         Keywords:
@@ -69,10 +76,6 @@ class NotebookHandler(CodeHandler):
         nbformat = import_required('nbformat', 'The Bokeh notebook application handler requires Jupyter Notebook to be installed.')
         nbconvert = import_required('nbconvert', 'The Bokeh notebook application handler requires Jupyter Notebook to be installed.')
 
-        if 'filename' not in kwargs:
-            raise ValueError('Must pass a filename to NotebookHandler')
-
-
         class StripMagicsProcessor(nbconvert.preprocessors.Preprocessor):
             """
             Preprocessor to convert notebooks to Python source while stripping
@@ -81,11 +84,11 @@ class NotebookHandler(CodeHandler):
 
             _magic_pattern = re.compile(r'^\s*(?P<magic>%%\w\w+)($|(\s+))')
 
-            def strip_magics(self, source):
+            def strip_magics(self, source: str) -> str:
                 """
                 Given the source of a cell, filter out all cell and line magics.
                 """
-                filtered=[]
+                filtered: List[str] = []
                 for line in source.splitlines():
                     match = self._magic_pattern.match(line)
                     if match is None:
@@ -106,8 +109,7 @@ class NotebookHandler(CodeHandler):
                 self._cell_counter = 0
                 return self.preprocess(nb,resources)
 
-        preprocessors=[StripMagicsProcessor()]
-        filename = kwargs['filename']
+        preprocessors = [StripMagicsProcessor()]
 
         with open(filename, encoding="utf-8") as f:
             nb = nbformat.read(f, nbformat.NO_CONVERT)
@@ -120,9 +122,7 @@ class NotebookHandler(CodeHandler):
             source = source.replace('get_ipython().run_line_magic', '')
             source = source.replace('get_ipython().magic', '')
 
-            kwargs['source'] = source
-
-        super().__init__(*args, **kwargs)
+        super().__init__(source=source, filename=filename, argv=argv, package=package)
 
 #-----------------------------------------------------------------------------
 # Private API
