@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2022, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -11,6 +11,8 @@
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
@@ -20,10 +22,15 @@ log = logging.getLogger(__name__)
 
 # Standard library imports
 import colorsys
+from typing import TYPE_CHECKING
 
 # Bokeh imports
+from ..util.deprecation import deprecated
 from .color import Color
 
+if TYPE_CHECKING:
+    from .rgb import RGB
+
 #-----------------------------------------------------------------------------
 # Globals and constants
 #-----------------------------------------------------------------------------
@@ -42,9 +49,13 @@ class HSL(Color):
     Alpha values may also optionally be provided. Otherwise, alpha values
     default to 1.
 
+    .. warning::
+        HSL is deprecated as of Bokeh 2.3.1 and will be removed in a future
+        release. Use RGB or named colors instead.
+
     '''
 
-    def __init__(self, h, s, l, a=1.0):
+    def __init__(self, h: float, s: float, l: float, a: float = 1.0) -> None:
         '''
 
         Args:
@@ -61,12 +72,13 @@ class HSL(Color):
                 An alpha value for this color in [0, 1] (default: 1.0)
 
         '''
+        deprecated((2, 3, 1), "HSL()", "RGB() or named colors")
         self.h = h
         self.s = s
         self.l = l
         self.a = a
 
-    def copy(self):
+    def copy(self) -> HSL:
         ''' Return a copy of this color value.
 
         Returns:
@@ -76,7 +88,7 @@ class HSL(Color):
         return HSL(self.h, self.s, self.l, self.a)
 
     @classmethod
-    def from_hsl(cls, value):
+    def from_hsl(cls, value: HSL) -> HSL:
         ''' Copy an HSL color from another HSL color value.
 
         Args:
@@ -90,7 +102,7 @@ class HSL(Color):
         return value.copy()
 
     @classmethod
-    def from_rgb(cls, value):
+    def from_rgb(cls, value: RGB) -> HSL:
         ''' Create an HSL color from an RGB color value.
 
         Args:
@@ -103,7 +115,7 @@ class HSL(Color):
         '''
         return value.to_hsl()
 
-    def to_css(self):
+    def to_css(self) -> str:
         ''' Generate the CSS representation of this HSL color.
 
         Returns:
@@ -111,11 +123,11 @@ class HSL(Color):
 
         '''
         if self.a == 1.0:
-            return "hsl(%d, %s%%, %s%%)" % (self.h, self.s*100, self.l*100)
+            return f"hsl({self.h}, {self.s*100}%, {self.l*100}%)"
         else:
-            return "hsla(%d, %s%%, %s%%, %s)" % (self.h, self.s*100, self.l*100, self.a)
+            return f"hsla({self.h}, {self.s*100}%, {self.l*100}%, {self.a})"
 
-    def to_hsl(self):
+    def to_hsl(self) -> HSL:
         ''' Return a HSL copy for this HSL color.
 
         Returns:
@@ -124,7 +136,7 @@ class HSL(Color):
         '''
         return self.copy()
 
-    def to_rgb(self):
+    def to_rgb(self) -> RGB:
         ''' Return a corresponding :class:`~bokeh.colors.rgb.RGB` color for
         this HSL color.
 
@@ -132,7 +144,7 @@ class HSL(Color):
             :class:`~bokeh.colors.rgb.RGB`
 
         '''
-        from .rgb import RGB # prevent circular import
+        from .rgb import RGB  # prevent circular import
         r, g, b = colorsys.hls_to_rgb(float(self.h)/360, self.l, self.s)
         return RGB(round(r*255), round(g*255), round(b*255), self.a)
 
