@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2022, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -33,6 +33,8 @@ The following are equivalent:
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
@@ -43,7 +45,7 @@ log = logging.getLogger(__name__)
 # Standard library imports
 import argparse
 import sys
-from typing import List
+from typing import Sequence
 
 # Bokeh imports
 from bokeh import __version__
@@ -65,8 +67,8 @@ __all__ = (
 #-----------------------------------------------------------------------------
 # General API
 #-----------------------------------------------------------------------------
-# TODO - struggling to resolve argv mypy typing. is nested tuple of ((arg_name, arg_dict), ...) not Sequence[str]
-def main(argv: List[str]) -> None:
+
+def main(argv: Sequence[str]) -> None:
     ''' Execute the Bokeh command.
 
     Args:
