@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2022, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -98,7 +98,7 @@ class Either(ParameterizedProperty):
             except ValueError:
                 pass
 
-        raise ValueError("Could not transform {value!r}")
+        raise ValueError(f"Could not transform {value!r}")
 
     def validate(self, value, detail=True):
         super().validate(value, detail)
