@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2022, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -78,7 +78,7 @@ def get_env():
         # Non-frozen Python and cx_Freeze can use __file__ directly
         templates_path = join(dirname(__file__), '_templates')
 
-    return Environment(loader=FileSystemLoader(templates_path))
+    return Environment(loader=FileSystemLoader(templates_path), trim_blocks=True, lstrip_blocks=True)
 
 #-----------------------------------------------------------------------------
 # Private API
