@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2022, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -11,6 +11,8 @@
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
@@ -26,9 +28,14 @@ import_required("selenium.webdriver",
 # Standard library imports
 import atexit
 import os
-from os.path import devnull, dirname, isfile, join
+from os.path import (
+    devnull,
+    dirname,
+    isfile,
+    join,
+)
 from shutil import which
-from typing import List, Optional
+from typing import List, Set
 
 # External imports
 from selenium import webdriver
@@ -74,20 +81,20 @@ def create_firefox_webdriver() -> WebDriver:
     else:
         binary_path = firefox
 
-    from selenium.webdriver.firefox.firefox_binary import FirefoxBinary
+    from selenium.webdriver.firefox.firefox_binary import FirefoxBinary  # type: ignore [import]
     binary = FirefoxBinary(binary_path)
 
     options = webdriver.firefox.options.Options()
     options.add_argument("--headless")
 
-    return webdriver.Firefox(
+    return webdriver.Firefox(  # type: ignore [attr-defined]
         options=options,
         firefox_binary=binary,
         executable_path=geckodriver,
         service_log_path=devnull,
     )
 
-def create_chromium_webdriver(extra_options: Optional[List[str]] = None) -> WebDriver:
+def create_chromium_webdriver(extra_options: List[str] | None = None) -> WebDriver:
     options = webdriver.chrome.options.Options()
     options.add_argument("--headless")
     options.add_argument("--hide-scrollbars")
@@ -96,7 +103,7 @@ def create_chromium_webdriver(extra_options: Optional[List[str]] = None) -> WebD
     if extra_options:
         for op in extra_options:
             options.add_argument(op)
-    return webdriver.Chrome(options=options)
+    return webdriver.Chrome(options=options)  # type: ignore [attr-defined]
 
 #-----------------------------------------------------------------------------
 # Private API
@@ -105,30 +112,27 @@ def create_chromium_webdriver(extra_options: Optional[List[str]] = None) -> WebD
 def _is_executable(path: str) -> bool:
     return isfile(path) and os.access(path, os.X_OK)
 
-def _try_create_firefox_webdriver() -> Optional[WebDriver]:
+def _try_create_firefox_webdriver() -> WebDriver | None:
     try:
         return create_firefox_webdriver()
     except Exception:
         return None
 
-def _try_create_chromium_webdriver() -> Optional[WebDriver]:
+def _try_create_chromium_webdriver() -> WebDriver | None:
     try:
         return create_chromium_webdriver()
     except Exception:
         return None
 
 class _WebdriverState:
-    '''
-
-    '''
 
     reuse: bool
-    kind: Optional[DriverKind]
+    kind: DriverKind | None
 
-    current: Optional[WebDriver]
-    _drivers: List[WebDriver]
+    current: WebDriver | None
+    _drivers: Set[WebDriver]
 
-    def __init__(self, *, kind: Optional[DriverKind] = None, reuse: bool = True):
+    def __init__(self, *, kind: DriverKind | None = None, reuse: bool = True) -> None:
         self.kind = kind
         self.reuse = reuse
         self.current = None
@@ -149,12 +153,12 @@ class _WebdriverState:
             self.current = self.create()
         return self.current
 
-    def create(self, kind: Optional[DriverKind] = None) -> WebDriver:
+    def create(self, kind: DriverKind | None = None) -> WebDriver:
         driver = self._create(kind)
         self._drivers.add(driver)
         return driver
 
-    def _create(self, kind: Optional[DriverKind]) -> WebDriver:
+    def _create(self, kind: DriverKind | None) -> WebDriver:
         driver_kind = kind or self.kind
 
         if driver_kind is None:
