@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2022, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -12,6 +12,8 @@ Bokeh plots
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
@@ -38,6 +40,7 @@ from ..core.properties import (
     Override,
     Seq,
     String,
+    TextLike,
     Tuple,
 )
 from ..core.property_mixins import ScalarLineProps, ScalarTextProps
@@ -121,13 +124,8 @@ class Axis(GuideRenderer):
     of ticks.
     """)
 
-    axis_label = Nullable(String, default="", help="""
-    A text label for the axis, displayed parallel to the axis rule.
-
-    .. note::
-        LaTeX notation is not currently supported; please see
-        :bokeh-issue:`647` to track progress or contribute.
-
+    axis_label = Nullable(TextLike, help="""
+    A text or LaTeX notation label for the axis, displayed parallel to the axis rule.
     """)
 
     axis_label_standoff = Int(default=5, help="""
@@ -135,8 +133,8 @@ class Axis(GuideRenderer):
     from the tick labels.
     """)
 
-    axis_label_props = Include(ScalarTextProps, help="""
-    The %s of the axis label.
+    axis_label_props = Include(ScalarTextProps, prefix="axis_label", help="""
+    The {prop} of the axis label.
     """)
 
     axis_label_text_font_size = Override(default="13px")
@@ -153,7 +151,7 @@ class Axis(GuideRenderer):
     number is supplied, the angle of the text is measured from horizontal.
     """)
 
-    major_label_overrides = Dict(Either(Float, String), String, default={}, help="""
+    major_label_overrides = Dict(Either(Float, String), TextLike, default={}, help="""
     Provide explicit tick label values for specific tick locations that
     override normal formatting.
     """)
@@ -162,8 +160,8 @@ class Axis(GuideRenderer):
     Allows to filter out labels, e.g. declutter labels to avoid overlap.
     """)
 
-    major_label_props = Include(ScalarTextProps, help="""
-    The %s of the major tick labels.
+    major_label_props = Include(ScalarTextProps, prefix="major_label", help="""
+    The {prop} of the major tick labels.
     """)
 
     major_label_text_align = Override(default="center")
@@ -172,12 +170,12 @@ class Axis(GuideRenderer):
 
     major_label_text_font_size = Override(default="11px")
 
-    axis_props = Include(ScalarLineProps, help="""
-    The %s of the axis line.
+    axis_props = Include(ScalarLineProps, prefix="axis", help="""
+    The {prop} of the axis line.
     """)
 
-    major_tick_props = Include(ScalarLineProps, help="""
-    The %s of the major ticks.
+    major_tick_props = Include(ScalarLineProps,prefix="major_tick", help="""
+    The {prop} of the major ticks.
     """)
 
     major_tick_in = Int(default=2, help="""
@@ -190,8 +188,8 @@ class Axis(GuideRenderer):
     main plot area.
     """)
 
-    minor_tick_props = Include(ScalarLineProps, help="""
-    The %s of the minor ticks.
+    minor_tick_props = Include(ScalarLineProps, prefix="minor_tick", help="""
+    The {prop} of the minor ticks.
     """)
 
     minor_tick_in = Int(default=0, help="""
@@ -251,8 +249,8 @@ class CategoricalAxis(Axis):
 
     formatter = Override(default=lambda: CategoricalTickFormatter())
 
-    separator_props = Include(ScalarLineProps, help="""
-    The %s of the separator line between top-level categorical groups.
+    separator_props = Include(ScalarLineProps, prefix="separator", help="""
+    The {prop} of the separator line between top-level categorical groups.
 
     This property always applies to factors in the outermost level of nesting.
     """)
@@ -260,8 +258,8 @@ class CategoricalAxis(Axis):
     separator_line_color = Override(default="lightgrey")
     separator_line_width = Override(default=2)
 
-    group_props = Include(ScalarTextProps, help="""
-    The %s of the group categorical labels.
+    group_props = Include(ScalarTextProps, prefix="group", help="""
+    The {prop} of the group categorical labels.
 
     This property always applies to factors in the outermost level of nesting.
     If the list of categorical factors is flat (i.e. no nesting) then this
@@ -282,8 +280,8 @@ class CategoricalAxis(Axis):
     group_text_font_style = Override(default="bold")
     group_text_color = Override(default="grey")
 
-    subgroup_props = Include(ScalarTextProps, help="""
-    The %s of the subgroup categorical labels.
+    subgroup_props = Include(ScalarTextProps, prefix="subgroup", help="""
+    The {prop} of the subgroup categorical labels.
 
     This property always applies to factors in the middle level of nesting.
     If the list of categorical factors is has only zero or one levels of nesting,
@@ -324,7 +322,7 @@ class MercatorAxis(LinearAxis):
             (default: 'lat')
 
     '''
-    def __init__(self, dimension='lat', **kw):
+    def __init__(self, dimension='lat', **kw) -> None:
         super().__init__(**kw)
 
         # Just being careful. It would be defeat the purpose for anyone to actually
