@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2022, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -8,6 +8,8 @@
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
@@ -17,7 +19,16 @@ log = logging.getLogger(__name__)
 
 # Bokeh imports
 from ..core.has_props import abstract
-from ..core.properties import AnyRef, Bool, Int, NonNullable, Nullable, Seq, String, RestrictedDict
+from ..core.properties import (
+    AnyRef,
+    Bool,
+    Int,
+    NonNullable,
+    Nullable,
+    RestrictedDict,
+    Seq,
+    String,
+)
 from ..model import Model
 
 #-----------------------------------------------------------------------------
@@ -50,7 +61,7 @@ class IndexFilter(Filter):
     A list of integer indices representing the subset of data to select.
     """)
 
-    def __init__(self, *args, **kw):
+    def __init__(self, *args, **kw) -> None:
         if len(args) == 1 and "indices" not in kw:
             kw["indices"] = args[0]
 
@@ -65,7 +76,7 @@ class BooleanFilter(Filter):
     A list of booleans indicating which rows of data to select.
     """)
 
-    def __init__(self, *args, **kw):
+    def __init__(self, *args, **kw) -> None:
         if len(args) == 1 and "booleans" not in kw:
             kw["booleans"] = args[0]
 
@@ -84,7 +95,7 @@ class GroupFilter(Filter):
     The value of the column indicating the rows of data to keep.
     """)
 
-    def __init__(self, *args, **kw):
+    def __init__(self, *args, **kw) -> None:
         if len(args) == 2 and "column_name" not in kw and "group" not in kw:
             kw["column_name"] = args[0]
             kw["group"] = args[1]
