@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2022, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -1195,6 +1195,11 @@ class Scatter(Marker):
         |data units|, use the :class:`~bokeh.models.glyphs.Circle` glyph instead
         of the ``Scatter`` glyph.
 
+    .. note::
+        ``Scatter`` markers with multiple marker types may be drawn in a
+        different order when using the WebGL output backend. This is an explicit
+        trade-off made in the interests of performance.
+
     '''
 
     __example__ = "examples/reference/models/Scatter.py"
