@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2022, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -27,6 +27,7 @@ from .. import palettes
 from ..core.enums import Palette
 from ..core.has_props import abstract
 from ..core.properties import (
+    Bool,
     Color,
     Either,
     Enum,
@@ -263,6 +264,13 @@ class ScanningColorMapper(ContinuousColorMapper):
 class EqHistColorMapper(ScanningColorMapper):
     bins = Int(default=256*256, help="Number of histogram bins")
 
+    rescale_discrete_levels = Bool(default=False, help="""
+    If there are only a few discrete levels in the values that are color
+    mapped then ``rescale_discrete_levels=True`` decreases the lower limit of
+    the span so that the values are rendered towards the top end of the
+    palette.
+    """)
+
 #-----------------------------------------------------------------------------
 # Dev API
 #-----------------------------------------------------------------------------
