@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2022, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -13,6 +13,8 @@ and with options for "auto sizing".
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
@@ -46,7 +48,6 @@ from ..core.properties import (
 from ..core.validation import error
 from ..core.validation.errors import DUPLICATE_FACTORS
 from ..model import Model
-from .renderers import DataRenderer
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -132,7 +133,7 @@ class Range1d(Range):
     maximum visible interval. Can be a ``TimeDelta``. Note that ``bounds`` can
     impose an implicit constraint on the maximum interval as well. """)
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args, **kwargs) -> None:
         if args and ('start' in kwargs or 'end' in kwargs):
             raise ValueError("'start' and 'end' keywords cannot be used with positional arguments")
         if args and len(args) != 2:
@@ -161,7 +162,7 @@ class DataRange(Range):
 
     """)
 
-    renderers = Either(List(Instance(DataRenderer)), Auto, help="""
+    renderers = Either(List(Instance(Model)), Auto, help="""
     An explicit list of renderers to autorange against. If unset,
     defaults to all renderers on a plot.
     """)
@@ -271,7 +272,7 @@ class DataRange1d(DataRange):
     bounds computations.
     """)
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args, **kwargs) -> None:
         if kwargs.get('follow') is not None:
             kwargs['bounds'] = None
         super().__init__(**kwargs)
@@ -452,7 +453,7 @@ class FactorRange(Range):
     padding properties and whether or not factors are grouped.
     """)
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args, **kwargs) -> None:
         if args and "factors" in kwargs:
             raise ValueError("'factors' keyword cannot be used with positional arguments")
         elif args:
