@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2022, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -236,7 +236,7 @@ class Tap(GestureTool):
 
 @abstract
 class SelectTool(GestureTool):
-    ''' A base class for tools that perfrom "selections", e.g. ``BoxSelectTool``.
+    ''' A base class for tools that perform "selections", e.g. ``BoxSelectTool``.
 
     '''
 
