@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2022, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -8,6 +8,8 @@
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
@@ -19,13 +21,13 @@ log = logging.getLogger(__name__)
 import itertools
 import re
 import warnings
+from dataclasses import dataclass
 from typing import (
     Any,
+    Callable,
     Dict,
     Iterator,
     List,
-    NamedTuple,
-    Optional,
     Sequence,
     Tuple,
     Union,
@@ -36,8 +38,19 @@ from typing import (
 from typing_extensions import Literal
 
 # Bokeh imports
-from ..models import HoverTool, Plot, Tool, Toolbar
-from ..models.tools import Drag, InspectTool, Scroll, Tap
+from ..models import (
+    HoverTool,
+    Plot,
+    Tool,
+    Toolbar,
+)
+from ..models.tools import (
+    Drag,
+    GestureTool,
+    InspectTool,
+    Scroll,
+    Tap,
+)
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -62,18 +75,21 @@ ActiveDrag = Union[Drag, Auto, str, None]
 ActiveInspect = Union[List[InspectTool], InspectTool, Auto, str, None]
 ActiveScroll = Union[Scroll, Auto, str, None]
 ActiveTap = Union[Tap, Auto, str, None]
+ActiveMulti = Union[GestureTool, Auto, str, None]
 
 def process_active_tools(toolbar: Toolbar, tool_map: Dict[str, Tool],
-        active_drag: ActiveDrag, active_inspect: ActiveInspect, active_scroll: ActiveScroll, active_tap: ActiveTap) -> None:
+        active_drag: ActiveDrag, active_inspect: ActiveInspect, active_scroll: ActiveScroll,
+        active_tap: ActiveTap, active_multi: ActiveMulti) -> None:
     """ Adds tools to the plot object
 
     Args:
         toolbar (Toolbar): instance of a Toolbar object
         tools_map (dict[str]): tool_map from _process_tools_arg
-        active_drag (str or Tool): the tool to set active for drag
-        active_inspect (str or Tool): the tool to set active for inspect
-        active_scroll (str or Tool): the tool to set active for scroll
-        active_tap (str or Tool): the tool to set active for tap
+        active_drag (str, None, "auto" or Tool): the tool to set active for drag
+        active_inspect (str, None, "auto", Tool or Tool[]): the tool to set active for inspect
+        active_scroll (str, None, "auto" or Tool): the tool to set active for scroll
+        active_tap (str, None, "auto" or Tool): the tool to set active for tap
+        active_multi (str, None, "auto" or Tool): the tool to set active for tap
 
     Returns:
         None
@@ -81,12 +97,12 @@ def process_active_tools(toolbar: Toolbar, tool_map: Dict[str, Tool],
     Note:
         This function sets properties on Toolbar
     """
-    if active_drag in ['auto', None] or isinstance(active_drag, Tool):
+    if active_drag in ["auto", None] or isinstance(active_drag, Tool):
         toolbar.active_drag = cast(Any, active_drag)
     elif active_drag in tool_map:
         toolbar.active_drag = cast(Any, tool_map[active_drag])
     else:
-        raise ValueError("Got unknown %r for 'active_drag', which was not a string supplied in 'tools' argument" % active_drag)
+        raise ValueError(f"Got unknown {active_drag!r} for 'active_drag', which was not a string supplied in 'tools' argument")
 
     if active_inspect in ["auto", None] or isinstance(active_inspect, Tool) or \
             (isinstance(active_inspect, list) and all(isinstance(t, Tool) for t in active_inspect)):
@@ -94,24 +110,31 @@ def process_active_tools(toolbar: Toolbar, tool_map: Dict[str, Tool],
     elif isinstance(active_inspect, str) and active_inspect in tool_map:
         toolbar.active_inspect = cast(Any, tool_map[active_inspect])
     else:
-        raise ValueError("Got unknown %r for 'active_inspect', which was not a string supplied in 'tools' argument" % active_scroll)
+        raise ValueError(f"Got unknown {active_inspect!r} for 'active_inspect', which was not a string supplied in 'tools' argument")
 
-    if active_scroll in ['auto', None] or isinstance(active_scroll, Tool):
+    if active_scroll in ["auto", None] or isinstance(active_scroll, Tool):
         toolbar.active_scroll = cast(Any, active_scroll)
     elif active_scroll in tool_map:
         toolbar.active_scroll = cast(Any, tool_map[active_scroll])
     else:
-        raise ValueError("Got unknown %r for 'active_scroll', which was not a string supplied in 'tools' argument" % active_scroll)
+        raise ValueError(f"Got unknown {active_scroll!r} for 'active_scroll', which was not a string supplied in 'tools' argument")
 
-    if active_tap in ['auto', None] or isinstance(active_tap, Tool):
+    if active_tap in ["auto", None] or isinstance(active_tap, Tool):
         toolbar.active_tap = cast(Any, active_tap)
     elif active_tap in tool_map:
         toolbar.active_tap = cast(Any, tool_map[active_tap])
     else:
-        raise ValueError("Got unknown %r for 'active_tap', which was not a string supplied in 'tools' argument" % active_tap)
+        raise ValueError(f"Got unknown {active_tap!r} for 'active_tap', which was not a string supplied in 'tools' argument")
 
-def process_tools_arg(plot: Plot, tools: Union[str, Sequence[Union[Tool, str]]],
-        tooltips: Optional[Union[str, Tuple[str, str]]] = None) -> Tuple[List[Tool], Dict[str, Tool]]:
+    if active_multi in ["auto", None] or isinstance(active_multi, Tool):
+        toolbar.active_multi = cast(Any, active_multi)
+    elif active_multi in tool_map:
+        toolbar.active_multi = cast(Any, tool_map[active_multi])
+    else:
+        raise ValueError(f"Got unknown {active_multi!r} for 'active_multi', which was not a string supplied in 'tools' argument")
+
+def process_tools_arg(plot: Plot, tools: str | Sequence[Tool | str],
+        tooltips: str | Tuple[str, str] | None = None) -> Tuple[List[Tool], Dict[str, Tool]]:
     """ Adds tools to the plot object
 
     Args:
@@ -130,15 +153,15 @@ def process_tools_arg(plot: Plot, tools: Union[str, Sequence[Union[Tool, str]]],
 
     repeated_tools = [ str(obj) for obj in _collect_repeated_tools(tool_objs) ]
     if repeated_tools:
-        warnings.warn("%s are being repeated" % ",".join(repeated_tools))
+        warnings.warn(f"{','.join(repeated_tools)} are being repeated")
 
     if tooltips is not None:
         for tool_obj in tool_objs:
             if isinstance(tool_obj, HoverTool):
-                tool_obj.tooltips = tooltips
+                tool_obj.tooltips = tooltips # type: ignore
                 break
         else:
-            tool_objs.append(HoverTool(tooltips=tooltips)) # type: ignore
+            tool_objs.append(HoverTool(tooltips=tooltips))
 
     return tool_objs, tool_map
 
@@ -146,9 +169,9 @@ def process_tools_arg(plot: Plot, tools: Union[str, Sequence[Union[Tool, str]]],
 # Private API
 #-----------------------------------------------------------------------------
 
-def _resolve_tools(tools: Union[str, Sequence[Union[Tool, str]]]) -> Tuple[List[Tool], Dict[str, Tool]]:
-    tool_objs = []
-    tool_map = {}
+def _resolve_tools(tools: str | Sequence[Tool | str]) -> Tuple[List[Tool], Dict[str, Tool]]:
+    tool_objs: List[Tool] = []
+    tool_map: Dict[str, Tool] = {}
 
     if not isinstance(tools, str):
         temp_tool_str = ""
@@ -173,11 +196,12 @@ def _resolve_tools(tools: Union[str, Sequence[Union[Tool, str]]]) -> Tuple[List[
     return tool_objs, tool_map
 
 def _collect_repeated_tools(tool_objs: List[Tool]) -> Iterator[Tool]:
-    class Item(NamedTuple):
+    @dataclass
+    class Item:
         obj: Tool
         properties: Dict[str, Any]
 
-    key = lambda obj: obj.__class__.__name__
+    key: Callable[[Tool], str] = lambda obj: obj.__class__.__name__
 
     for _, group in itertools.groupby(sorted(tool_objs, key=key), key=key):
         rest = [ Item(obj, obj.properties_with_values()) for obj in group ]
