@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2022, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -946,6 +946,11 @@ Examples:
             from a data source column, *all* markers including circles may only
             be configured with ``size`` in |screen units|.
 
+        .. note::
+            ``Scatter`` markers with multiple marker types may be drawn in a
+            different order when using the WebGL output backend. This is an explicit
+            trade-off made in the interests of performance.
+
         '''
         marker_type = kwargs.pop("marker", "circle")
 
