@@ -1,25 +1,31 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2022, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
 #-----------------------------------------------------------------------------
-''' Provide Fisher's Iris flower data set. This module has an attribute
-``flowers`` which is a pandas DataFrame with the following fields:
+''' Provide `Fisher's Iris dataset`_.
 
-.. code-block:: python
+This module contains one pandas Dataframe: ``flowers``.
 
-    flowers['petal_length']
-    flowers['petal_width']
-    flowers['sepal_length']
-    flowers['sepal_width']
-    flowers['species']
+.. note::
+    This sampledata is maintained for historical compatibility. Please consider
+    `alternatives to Iris`_ such as :ref:`sampledata_penguins`.
+
+.. rubric:: ``flowers``
+
+:bokeh-dataframe:`bokeh.sampledata.iris.flowers`
+
+.. _Fisher's Iris dataset: https://en.wikipedia.org/wiki/Iris_flower_data_se
+.. _alternatives to Iris: https://www.meganstodel.com/posts/no-to-iris/
 
 '''
 
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
