@@ -1,15 +1,14 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2022, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
 #-----------------------------------------------------------------------------
-''' Provide recorded stock data for the following stocks:
+''' Provide historical ticker data for selected stocks.
 
-    AAPL, FB, GOOG, IBM, MSFT
+This module contains five dicts: ``AAPL``, ``FB``, ``GOOG``, ``IBM``, and ``MSFT``.
 
-Each eries is available as an attribute on the module (e.g., ``stocks.AAPL``)
-and the value is a dictionary with the structure:
+Each dictionary has the structure:
 
 .. code-block:: python
 
@@ -26,6 +25,8 @@ and the value is a dictionary with the structure:
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
