@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2022, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -204,7 +204,7 @@ class BaseServer:
             self.start()
         # Install shutdown hooks
         atexit.register(self._atexit)
-        signal.signal(signal.SIGTERM, self._sigterm)
+        signal.signal(signal.SIGTERM, self._sigterm)   # type: ignore[arg-type]
         try:
             self._loop.start()
         except KeyboardInterrupt:
