@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2022, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -11,6 +11,8 @@
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
+from __future__ import annotations
+
 import logging # isort:skip
 log = logging.getLogger(__name__)
 
@@ -20,7 +22,7 @@ log = logging.getLogger(__name__)
 
 # Standard library imports
 import os
-from typing import Dict, Optional
+from typing import Dict
 
 # External imports
 from tornado.web import HTTPError, StaticFileHandler
@@ -60,7 +62,7 @@ class MultiRootStaticHandler(StaticFileHandler):
         else:
             raise HTTPError(404)
 
-    def validate_absolute_path(self, root: str, absolute_path: str) -> Optional[str]:
+    def validate_absolute_path(self, root: str, absolute_path: str) -> str | None:
         for name, artifacts_dir in root.items():
             if absolute_path.startswith(artifacts_dir):
                 return super().validate_absolute_path(artifacts_dir, absolute_path)
