@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2022, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -11,9 +11,9 @@
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
+from __future__ import annotations
 
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
@@ -21,10 +21,12 @@ log = logging.getLogger(__name__)
 #-----------------------------------------------------------------------------
 
 # Standard library imports
-from os.path import abspath
 import webbrowser
+from os.path import abspath
+from typing import Dict, cast
 
 # External imports
+from typing_extensions import Literal, Protocol
 
 # Bokeh imports
 from ..settings import settings
@@ -33,7 +35,10 @@ from ..settings import settings
 # Globals and constants
 #-----------------------------------------------------------------------------
 
-NEW_PARAM = {'tab': 2, 'window': 1}
+BrowserTarget = Literal["same", "window", "tab"]
+TargetCode = Literal[0, 1, 2]
+
+NEW_PARAM: Dict[BrowserTarget, TargetCode] = {"same": 0, "window": 1, "tab": 2}
 
 __all__ = (
     'DummyWebBrowser',
@@ -45,25 +50,32 @@ __all__ = (
 # General API
 #-----------------------------------------------------------------------------
 
-class DummyWebBrowser(object):
+class BrowserLike(Protocol):
+    ''' Interface for browser-like objects.
+
+    '''
+    def open(self, url: str, new: TargetCode = ..., autoraise: bool = ...) -> bool:
+        ...
+
+class DummyWebBrowser:
     ''' A "no-op" web-browser controller.
 
     '''
-    def open(self, url, new=0, autoraise=True):
+    def open(self, url: str, new: TargetCode = 0, autoraise: bool = True) -> bool:
         ''' Receive standard arguments and take no action. '''
-        pass
+        return True
 
-def get_browser_controller(browser=None):
+def get_browser_controller(browser: str | None = None) -> BrowserLike:
     ''' Return a browser controller.
 
     Args:
         browser (str or None) : browser name, or ``None`` (default: ``None``)
             If passed the string ``'none'``, a dummy web browser controller
-            is returned
+            is returned.
 
             Otherwise, use the value to select an appropriate controller using
-            the ``webbrowser`` standard library module. In the value is
-            ``None`` then a system default is used.
+            the :doc:`webbrowser <python:library/webbrowser>` standard library
+            module. If the value is ``None``, a system default is used.
 
     Returns:
         controller : a web browser controller
@@ -71,56 +83,53 @@ def get_browser_controller(browser=None):
     '''
     browser = settings.browser(browser)
 
-    if browser is not None:
-        if browser == 'none':
-            controller = DummyWebBrowser()
-        else:
-            controller = webbrowser.get(browser)
+    if browser is None:
+        controller = cast(BrowserLike, webbrowser)
+    elif browser == "none":
+        controller = DummyWebBrowser()
     else:
-        controller = webbrowser
+        controller = webbrowser.get(browser)
 
     return controller
 
-def view(location, browser=None, new="same", autoraise=True):
-        ''' Open a browser to view the specified location.
+def view(location: str, browser: str | None = None, new: BrowserTarget = "same", autoraise: bool = True) -> None:
+    ''' Open a browser to view the specified location.
 
-        Args:
-            location (str) : Location to open
-                If location does not begin with "http:" it is assumed
-                to be a file path on the local filesystem.
-            browser (str or None) : what browser to use (default: None)
-                If ``None``, use the system default browser.
-            new (str) : How to open the location. Valid values are:
+    Args:
+        location (str) : Location to open
+            If location does not begin with "http:" it is assumed
+            to be a file path on the local filesystem.
+        browser (str or None) : what browser to use (default: None)
+            If ``None``, use the system default browser.
+        new (str) : How to open the location. Valid values are:
 
-                ``'same'`` - open in the current tab
+            ``'same'`` - open in the current tab
 
-                ``'tab'`` - open a new tab in the current window
+            ``'tab'`` - open a new tab in the current window
 
-                ``'window'`` - open in a new window
-            autoraise (bool) : Whether to automatically raise the location
-                in a new browser window (default: True)
+            ``'window'`` - open in a new window
+        autoraise (bool) : Whether to automatically raise the location
+            in a new browser window (default: True)
 
-        Returns:
-            None
+    Returns:
+        None
 
-        '''
-        try:
-            new = { "same": 0, "window": 1, "tab": 2 }[new]
-        except KeyError:
-            raise RuntimeError("invalid 'new' value passed to view: %r, valid values are: 'same', 'window', or 'tab'" % new)
+    '''
+    try:
+        new_id = NEW_PARAM[new]
+    except KeyError:
+        raise RuntimeError(f"invalid 'new' value passed to view: {new!r}, valid values are: 'same', 'window', or 'tab'")
 
-        if location.startswith("http"):
-            url = location
-        else:
-            url = "file://" + abspath(location)
+    if location.startswith("http"):
+        url = location
+    else:
+        url = "file://" + abspath(location)
 
-        try:
-            controller = get_browser_controller(browser)
-            controller.open(url, new=new, autoraise=autoraise)
-        except (SystemExit, KeyboardInterrupt):
-            raise
-        except:
-            pass
+    try:
+        controller = get_browser_controller(browser)
+        controller.open(url, new=new_id, autoraise=autoraise)
+    except Exception:
+        pass
 
 #-----------------------------------------------------------------------------
 # Dev API
